#include "gpu.h"
#include "gfx/renderer_api.h"
#include "runtime/config.h"
#include "runtime/threading.h"

#include <stdlib.h>
#include <string.h>

RT_CVAR_I(rt_VkMaxBufferCount, "Number of slots for gpu buffers. Default: 1024.", 1024);

typedef struct rt_buffer_data_s {
    VkBuffer buffer;
    VmaAllocation allocation;
    size_t size;
    rt_buffer_usage usage;
    rt_buffer_type type;

    rt_rwlock lock;

    struct rt_buffer_data_s *next_free;
} rt_buffer_data;

static rt_buffer_data *_buffers;
static rt_buffer_data *_first_free;
static rt_mutex *_list_lock;

rt_result InitBufferManagement(void) {
    size_t n = (size_t)rt_VkMaxBufferCount.i;
    _buffers = malloc(sizeof(rt_buffer_data) * n);
    if (!_buffers)
        return RT_OUT_OF_MEMORY;
    return RT_SUCCESS;
}

void ShutdownBufferManagement(void) {
    for (int i = 0; i < rt_VkMaxBufferCount.i; ++i) {
        if (_buffers[i].buffer == VK_NULL_HANDLE)
            continue;
        vmaDestroyBuffer(g_gpu.allocator, _buffers[i].buffer, _buffers[i].allocation);
        rtDestroyRWLock(&_buffers[i].lock);
        memset(&_buffers[i], 0, sizeof(_buffers[i]));
    }
    free(_buffers);
    _first_free = NULL;
    rtDestroyMutex(_list_lock);
}

void RT_RENDERER_API_FN(DestroyBuffers)(uint32_t count, rt_buffer_handle *buffers);

rt_result RT_RENDERER_API_FN(CreateBuffers)(uint32_t count,
                                            const rt_buffer_info *info,
                                            rt_buffer_handle *p_buffers) {
    for (uint32_t i = 0; i < count; ++i) {
        rtLockMutex(_list_lock);
        rt_buffer_data *slot = _first_free;
        if (!slot) {
            rtUnlockMutex(_list_lock);
            if (i > 0)
                rtRenDestroyBuffers(i, p_buffers);
            return RT_OUT_OF_MEMORY;
        }
        rtUnlockMutex(_list_lock);
    }
    return RT_SUCCESS;
}

void RT_RENDERER_API_FN(DestroyBuffers)(uint32_t count, rt_buffer_handle *buffers) {

}
